@@ -127,7 +127,6 @@ import tarfile
 import tempfile
 import time
 from uuid import uuid4
-import sys
 
 import yaml
 
@@ -140,7 +139,6 @@ from lava_dispatcher.signals import SignalDirector
 from lava_dispatcher import utils
 
 from lava_dispatcher.actions import BaseAction
-from lava_dispatcher.device.target import Target
 from lava_dispatcher.downloader import download_image
 from lava_dispatcher.errors import GeneralError, CriticalError
 
@@ -542,6 +540,22 @@ class URLTestDefinition(object):
             f.write('set -ex\n')
             f.write('cd %s\n' % targetdir)
 
+            if self.skip_install != 'keys':
+                distro = self.context.client.target_device.deployment_data[
+                    'distro']
+                sources = self.testdef['install'].get('keys', [])
+                for src in sources:
+                    f.write('lava-add-keys %s' % src)
+                    f.write('\n')
+
+            if self.skip_install != 'sources':
+                distro = self.context.client.target_device.deployment_data[
+                    'distro']
+                sources = self.testdef['install'].get('sources', [])
+                for src in sources:
+                    f.write('lava-add-sources %s' % src)
+                    f.write('\n')
+
             if self.skip_install != 'deps':
                 distro = self.context.client.target_device.deployment_data['distro']
 
@@ -699,6 +713,8 @@ class cmd_lava_test_shell(BaseAction):
             'timeout': {'type': 'integer', 'optional': True},
             'role': {'type': 'string', 'optional': True},
             'skip_install': {'type': 'string', 'optional': True},
+            'lava_test_dir': {'type': 'string', 'optional': True},
+            'lava_test_results_dir': {'type': 'string', 'optional': True},
         },
         'additionalProperties': False,
     }
@@ -714,11 +730,14 @@ class cmd_lava_test_shell(BaseAction):
             'format': 'Dashboard Bundle Format 1.7',
         }
 
-    def run(self, testdef_urls=None, testdef_repos=None, timeout=-1, skip_install=None):
+    def run(self, testdef_urls=None, testdef_repos=None, timeout=-1, skip_install=None,
+            lava_test_dir=None, lava_test_results_dir=None):
         target = self.client.target_device
 
         delay = target.config.test_shell_serial_delay_ms
 
+        self._amend_test_execution_paths(target, lava_test_dir, lava_test_results_dir)
+
         testdef_objs = self._configure_target(target, testdef_urls,
                                               testdef_repos, skip_install)
 
@@ -987,7 +1006,7 @@ class cmd_lava_test_shell(BaseAction):
                 # lava/results-XXXXXXXXXX for post-mortem analysis
                 timestamp = datetime.now().strftime("%s")
                 os.rename(results_dir, results_dir + '-' + timestamp)
-                os.mkdir(results_dir)
+                utils.ensure_directory(results_dir)
         except Exception as e:
             if filesystem_access_failure:
                 # a failure when accessing the filesystem means the device
@@ -1054,3 +1073,34 @@ class cmd_lava_test_shell(BaseAction):
         test_result = parse_testcase_result(data,
                                             self._current_testdef.fixupdict)
         self._current_test_run['test_results'].append(test_result)
+
+    def _amend_test_execution_paths(self, target, lava_test_dir, lava_test_results_dir):
+        if lava_test_dir is not None:
+            # Preserve the default configuration value
+            if target.config.lava_test_dir_backup is None:
+                target.config.lava_test_dir_backup = target.lava_test_dir
+                logging.info("Preserving default test directory: %s" % target.config.lava_test_dir_backup)
+            # Set the user specified value
+            logging.info("Using user specified test directory: %s" % lava_test_dir)
+            target.config.lava_test_dir = lava_test_dir
+        else:
+            if target.config.lava_test_dir_backup is not None:
+                # Restore the default configuration value
+                target.config.lava_test_dir = target.config.lava_test_dir_backup
+                logging.info("Restoring default test directory: %s" % target.config.lava_test_dir_backup)
+                target.config.lava_test_dir_backup = None
+
+        if lava_test_results_dir is not None:
+            # Preserve the default configuration value
+            if target.config.lava_test_results_dir_backup is None:
+                target.config.lava_test_results_dir_backup = target.lava_test_results_dir
+                logging.info("Preserving default test results directory: %s" % target.config.lava_test_results_dir_backup)
+            # Set the user specified value
+            logging.info("Using user specified test results directory: %s" % lava_test_results_dir)
+            target.config.lava_test_results_dir = lava_test_results_dir
+        else:
+            if target.config.lava_test_results_dir_backup is not None:
+                # Reset the default configuration value
+                target.config.lava_test_results_dir = target.config.lava_test_results_dir_backup
+                logging.info("Restoring default test results directory: %s" % target.config.lava_test_results_dir_backup)
+                target.config.lava_test_results_dir_backup = None
