@@ -45,7 +45,6 @@ class DeviceSchema(schema.Schema):
     boot_cmds = schema.StringOption(fatal=True)  # Can do better here
     boot_cmds_android = schema.StringOption(fatal=True)  # And here
     boot_cmds_oe = schema.StringOption(fatal=True)  # And here?
-    boot_cmds_master = schema.StringOption()  # Optional
     read_boot_cmds_from_image = schema.BoolOption(default=True)
     boot_options = schema.ListOption()
     boot_linaro_timeout = schema.IntOption(default=300)
@@ -53,6 +52,8 @@ class DeviceSchema(schema.Schema):
     boot_part_android_org = schema.IntOption()
     boot_retries = schema.IntOption(default=3)
     bootloader_prompt = schema.StringOption()
+    bootloader_timeout = schema.IntOption(default=120)
+    image_boot_msg_timeout = schema.IntOption(default=120)
     send_char = schema.BoolOption(default=True)
     test_image_prompts = schema.ListOption(default=["\(initramfs\)",
                                                     "linaro-test",
@@ -114,6 +115,8 @@ class DeviceSchema(schema.Schema):
     tester_rc_cmd = schema.StringOption(null=True)
     lava_test_dir = schema.StringOption(null=True)
     lava_test_results_dir = schema.StringOption(null=True)
+    lava_test_dir_backup = schema.StringOption(default=None)
+    lava_test_results_dir_backup = schema.StringOption(default=None)
     val = schema.StringOption()
     sdcard_mountpoint_path = schema.StringOption(default="/storage/sdcard0")
     possible_partitions_files = schema.ListOption(default=["init.partitions.rc",
@@ -122,6 +125,7 @@ class DeviceSchema(schema.Schema):
     boot_files = schema.ListOption(default=['boot.txt', 'uEnv.txt'])
     boot_device = schema.IntOption(fatal=True)
     testboot_offset = schema.IntOption(fatal=True)
+    testboot_partition = schema.StringOption(default=None)
     # see doc/sdmux.rst for details
 
     lmp_default_id = schema.StringOption(default="13060000")
@@ -135,22 +139,28 @@ class DeviceSchema(schema.Schema):
 
     # for the HDMI module of LMP
     lmp_hdmi_id = schema.DictOption()
+    lmp_hdmi_defconf = schema.DictOption(default=None)
     lmp_hdmi_version = schema.StringOption(default="unknown")
 
     # for the SATA module of LMP
     lmp_sata_id = schema.DictOption()
+    lmp_sata_defconf = schema.DictOption(default=None)
     lmp_sata_version = schema.StringOption(default="unknown")
 
     # for the ETH module of LMP
     lmp_eth_id = schema.DictOption()
+    lmp_eth_defconf = schema.DictOption(default=None)
     lmp_eth_version = schema.StringOption(default="unknown")
 
     # for the LSGPIO module of LMP
     lmp_lsgpio_id = schema.DictOption()
+    lmp_lsgpio_defconf = schema.DictOption(default=None)
+    lmp_audio_defconf = schema.DictOption(default=None)
     lmp_lsgpio_version = schema.StringOption(default="unknown")
 
     # for the USB module of LMP
     lmp_usb_id = schema.DictOption()
+    lmp_usb_defconf = schema.DictOption(default=None)
     lmp_usb_version = schema.StringOption(default="unknown")
 
     # auto image login
@@ -160,25 +170,6 @@ class DeviceSchema(schema.Schema):
     password = schema.StringOption(default=None)
     login_commands = schema.ListOption(default=None)
 
-    simulator_version_command = schema.StringOption()
-    simulator_command = schema.StringOption()
-    simulator_command_flag = schema.StringOption(default=" -C ")
-    simulator_axf_files = schema.ListOption()
-    simulator_kernel_files = schema.ListOption(default=None)
-    simulator_kernel = schema.StringOption(default=None)
-    simulator_initrd_files = schema.ListOption(default=None)
-    simulator_initrd = schema.StringOption(default=None)
-    simulator_dtb_files = schema.ListOption(default=None)
-    simulator_dtb = schema.StringOption(default=None)
-    simulator_uefi_files = schema.ListOption(default=None)
-    simulator_bl1_files = schema.ListOption(default=None)
-    simulator_bl1 = schema.StringOption(default=None)
-    simulator_bl2_files = schema.ListOption(default=None)
-    simulator_bl2 = schema.StringOption(default=None)
-    simulator_bl31_files = schema.ListOption(default=None)
-    simulator_bl31 = schema.StringOption(default=None)
-    simulator_boot_wrapper = schema.StringOption(default=None)
-
     android_disable_suspend = schema.BoolOption(default=True)
     android_adb_over_usb = schema.BoolOption(default=False)
     android_adb_over_tcp = schema.BoolOption(default=True)
@@ -192,38 +183,55 @@ class DeviceSchema(schema.Schema):
     android_lava_block_device = schema.StringOption(default="mmcblk0")
     partition_padding_string_org = schema.StringOption(default="p")
     partition_padding_string_android = schema.StringOption(default="p")
+    android_serialno_patterns = schema.ListOption(default=['[0-9A-Fa-f]{16}'])
 
     arm_probe_binary = schema.StringOption(default='/usr/local/bin/arm-probe')
     arm_probe_config = schema.StringOption(default='/usr/local/etc/arm-probe-config')
     arm_probe_channels = schema.ListOption(default=['VDD_VCORE1'])
 
-    uefi_image_filename = schema.StringOption(default=None)
     customize = schema.DictOption(default=None)
-    vexpress_uefi_path = schema.StringOption(default=None)
-    vexpress_uefi_backup_path = schema.StringOption(default=None)
-    vexpress_stop_autoboot_prompt = schema.StringOption(
-        default='Press Enter to stop auto boot...')
-    vexpress_usb_mass_storage_device = schema.StringOption(default=None)
-
-    bl1_image_files = schema.ListOption(default=None)
-    bl1_image_filename = schema.StringOption(default="bl1.bin")
-    fip_image_files = schema.ListOption(default=None)
-    fip_image_filename = schema.StringOption(default="fip.bin")
-
-    wg_bl1_path = schema.StringOption(default="SOFTWARE/bl1.bin")
-    wg_fip_path = schema.StringOption(default="SOFTWARE/fip.bin")
-
-    wg_bl1_backup_path = schema.StringOption(default="SOFTWARE/backup-bl1.bin")
-    wg_fip_backup_path = schema.StringOption(default="SOFTWARE/backup-fip.bin")
-
-    wg_usb_mass_storage_device = schema.StringOption(default="/dev/disk/by-label/WG")
-    wg_stop_autoboot_prompt = schema.StringOption(
-        default='Press Enter to stop auto boot...')
 
     ecmeip = schema.StringOption()
     ipmi_power_sleep = schema.IntOption(default=1)
     ipmi_power_retries = schema.IntOption(default=10)
 
+    # for master devices
+    boot_cmds_master = schema.StringOption()
+    master_kernel = schema.StringOption()
+    master_ramdisk = schema.StringOption()
+    master_dtb = schema.StringOption()
+    master_firmware = schema.StringOption()
+    master_nfsrootfs = schema.StringOption()
+    # for master auto login
+    master_login_prompt = schema.StringOption(default=None)
+    master_password_prompt = schema.StringOption(default=None)
+    master_username = schema.StringOption(default=None)
+    master_password = schema.StringOption(default=None)
+    master_login_commands = schema.ListOption(default=None)
+
+    # for fastmodel devices
+    simulator_version_command = schema.StringOption()
+    simulator_command = schema.StringOption()
+    simulator_command_flag = schema.StringOption(default=" -C ")
+    simulator_axf_files = schema.ListOption()
+    simulator_kernel_files = schema.ListOption(default=None)
+    simulator_kernel = schema.StringOption(default=None)
+    simulator_initrd_files = schema.ListOption(default=None)
+    simulator_initrd = schema.StringOption(default=None)
+    simulator_dtb_files = schema.ListOption(default=None)
+    simulator_dtb = schema.StringOption(default=None)
+    simulator_uefi_files = schema.ListOption(default=None)
+    simulator_uefi_vars = schema.StringOption(default="uefi-vars.fd")
+    simulator_bl1_files = schema.ListOption(default=None)
+    simulator_bl1 = schema.StringOption(default=None)
+    simulator_bl2_files = schema.ListOption(default=None)
+    simulator_bl2 = schema.StringOption(default=None)
+    simulator_bl31_files = schema.ListOption(default=None)
+    simulator_bl31 = schema.StringOption(default=None)
+    simulator_boot_wrapper = schema.StringOption(default=None)
+    simulator_uefi_default = schema.StringOption(default=None)
+    simulator_bl1_default = schema.StringOption(default=None)
+
     # for dummy devices
     dummy_driver = schema.StringOption(default=None)
     dummy_schroot_chroot = schema.StringOption(default="default")
@@ -235,7 +243,6 @@ class DeviceSchema(schema.Schema):
     # for jtag devices
     jtag_driver = schema.StringOption(default=None)
     jtag_hard_reset_command = schema.StringOption(default=None)
-    jtag_hard_reset_sleep = schema.IntOption(default=60)
     # for stmc devices
     jtag_stmcconfig = schema.StringOption(default=None)
     jtag_stmc_ip = schema.StringOption(default=None)
@@ -251,6 +258,7 @@ class DeviceSchema(schema.Schema):
     fastboot_command = schema.StringOption()
     fastboot_kernel_load_addr = schema.StringOption()
     rootfs_partition = schema.StringOption(default='userdata')
+    start_fastboot_command = schema.StringOption(default='fastboot')
     shared_working_directory = schema.StringOption(default=None)
 
     # for bootloader devices
@@ -260,11 +268,31 @@ class DeviceSchema(schema.Schema):
     alternative_dir = schema.StringOption(default=None)
     u_load_addrs = schema.ListOption(default=None)
     z_load_addrs = schema.ListOption(default=None)
+    uimage_only = schema.BoolOption(default=False)
+    uimage_xip = schema.BoolOption(default=False)
+    append_dtb = schema.BoolOption(default=False)
 
     # for dynamic_vm devices
     dynamic_vm_backend_device_type = schema.StringOption(default='kvm')
     dynamic_vm_host = schema.StringOption(default=None)
 
+    # for vexpress devices
+    vexpress_uefi_default = schema.StringOption(default=None)
+    vexpress_bl1_default = schema.StringOption(default=None)
+    vexpress_uefi_path = schema.StringOption(default=None)
+    vexpress_uefi_image_files = schema.ListOption(default=None)
+    vexpress_uefi_image_filename = schema.StringOption(default="fip.bin")
+    vexpress_uefi_backup_path = schema.StringOption(default=None)
+    vexpress_bl1_path = schema.StringOption(default=None)
+    vexpress_bl1_image_files = schema.ListOption(default=None)
+    vexpress_bl1_image_filename = schema.StringOption(default="bl1.bin")
+    vexpress_bl1_backup_path = schema.StringOption(default=None)
+    vexpress_stop_autoboot_prompt = schema.StringOption(default='Press Enter to stop auto boot...')
+    vexpress_usb_mass_storage_device = schema.StringOption(default=None)
+    vexpress_requires_trusted_firmware = schema.BoolOption(default=False)
+    bridged_networking = schema.BoolOption(default=False)
+    bridge_interface = schema.StringOption(default="br0")
+
 
 class OptionDescriptor(object):
     def __init__(self, name):
