@@ -24,6 +24,8 @@ import cStringIO
 import logging
 import os
 import subprocess
+import signal
+import pexpect
 
 import lava_dispatcher.device.boot_options as boot_options
 
@@ -55,6 +57,7 @@ from lava_dispatcher.utils import (
     extract_ramdisk,
     create_ramdisk,
     ensure_directory,
+    touch,
 )
 from lava_dispatcher import deployment_data
 
@@ -75,6 +78,7 @@ class FastModelTarget(Target):
         self._dtb = None
         self._initrd = None
         self._uefi = None
+        self._uefi_vars = None
         self._bl1 = None
         self._bl2 = None
         self._bl31 = None
@@ -85,6 +89,7 @@ class FastModelTarget(Target):
         self._bootloadertype = 'u_boot'
         self._boot_tags = {}
         self._scratch_dir = self.scratch_dir
+        self._interface_name = None
 
     def _customize_android(self):
         self.deployment_data = deployment_data.android
@@ -122,6 +127,10 @@ class FastModelTarget(Target):
                 self._uefi = \
                     self._copy_first_find_from_list(subdir, odir,
                                                     self.config.simulator_uefi_files)
+                if self.config.simulator_uefi_vars and self._uefi_vars is None:
+                    # Create file for flashloader1
+                    self._uefi_vars = os.path.join(odir, self.config.simulator_uefi_vars)
+                    touch(self._uefi_vars)
 
         # These are common to both AXF and UEFI
         # Extract the kernel from the image
@@ -179,6 +188,8 @@ class FastModelTarget(Target):
             if self._uefi is None and self.config.simulator_uefi_files:
                 raise RuntimeError('No UEFI binary found, %r' %
                                    self.config.simulator_uefi_files)
+            if self._uefi_vars is None:
+                logging.warning('No uefi-vars.fd found')
 
         # These are common to both AXF and UEFI
         if self._sd_image is None:
@@ -190,26 +201,26 @@ class FastModelTarget(Target):
                                self.config.simulator_kernel_files)
         # Initrd is needed only for b.L models
         if self._initrd is None and self.config.simulator_initrd_files:
-            logging.warning('No INITRD found, %r' %
+            logging.warning('No INITRD found, %r',
                             self.config.simulator_initrd_files)
         # DTB is needed only for b.L models
         if self._dtb is None and self.config.simulator_dtb_files:
-            logging.warning('No DTB found, %r' %
+            logging.warning('No DTB found, %r',
                             self.config.simulator_dtb_files)
         # SECURE FLASHLOADERs are needed only for base and cortex models
         if self._bl1 is None and self.config.simulator_bl1_files:
             raise RuntimeError('No SECURE FLASHLOADER found, %r' %
                                self.config.simulator_bl1_files)
         if self._bl2 is None and self.config.simulator_bl2_files:
-            logging.warning('No SECURE FLASHLOADER found, %r' %
+            logging.warning('No SECURE FLASHLOADER found, %r',
                             self.config.simulator_bl2_files)
         if self._bl31 is None and self.config.simulator_bl31_files:
-            logging.warning('No SECURE FLASHLOADER found, %r' %
+            logging.warning('No SECURE FLASHLOADER found, %r',
                             self.config.simulator_bl31_files)
 
     def deploy_android(self, boot, system, data, rootfstype, bootloadertype,
                        target_type):
-        logging.info("Deploying Android on %s" % self.config.hostname)
+        logging.info("Deploying Android on %s", self.config.hostname)
 
         self._bootloadertype = bootloadertype
 
@@ -265,10 +276,6 @@ class FastModelTarget(Target):
             raise CriticalError("A ramdisk image is required")
         elif dtb is None:
             raise CriticalError("A dtb is required")
-        elif bootloader is None:
-            raise CriticalError("UEFI image is required")
-        elif bootloader is None:
-            raise CriticalError("BL1 firmware is required")
 
         if rootfs is not None or nfsrootfs is not None or firmware is not None:
             logging.warn("This platform only suports ramdisk booting, ignoring other parameters")
@@ -292,12 +299,28 @@ class FastModelTarget(Target):
         self._dtb = download_image(dtb, self.context, self._scratch_dir,
                                    decompress=False)
         self._boot_tags['{DTB}'] = os.path.relpath(self._dtb, self._scratch_dir)
-        self._uefi = download_image(bootloader, self.context, self._scratch_dir,
-                                    decompress=False)
-        self._bl1 = download_image(bl1, self.context, self._scratch_dir,
-                                   decompress=False)
 
         # Optional
+        if bootloader is None:
+            if self.config.simulator_uefi_default is None:
+                raise CriticalError("UEFI image is required")
+            else:
+                self._uefi = download_image(self.config.simulator_uefi_default, self.context,
+                                            self._scratch_dir, decompress=False)
+        else:
+            self._uefi = download_image(bootloader, self.context,
+                                        self._scratch_dir, decompress=False)
+
+        if bl1 is None:
+            if self.config.simulator_bl1_default is None:
+                raise CriticalError("BL1 firmware is required")
+            else:
+                self._bl1 = download_image(self.config.simulator_bl1_default, self.context,
+                                           self._scratch_dir, decompress=False)
+        else:
+            self._bl1 = download_image(bl1, self.context,
+                                       self._scratch_dir, decompress=False)
+
         if bl2 is not None:
             self._bl2 = download_image(bl2, self.context, self._scratch_dir,
                                        decompress=False)
@@ -305,6 +328,11 @@ class FastModelTarget(Target):
             self._bl31 = download_image(bl31, self.context, self._scratch_dir,
                                         decompress=False)
 
+        if self.config.simulator_uefi_vars and self._uefi_vars is None:
+            # Create file for flashloader1
+            self._uefi_vars = os.path.join(self._scratch_dir, self.config.simulator_uefi_vars)
+            touch(self._uefi_vars)
+
         # Get deployment data
         self.deployment_data = deployment_data.get(target_type)
 
@@ -340,7 +368,7 @@ class FastModelTarget(Target):
                 yield path
 
     def extract_tarball(self, tarball_url, partition, directory='/'):
-        logging.info('extracting %s to target' % tarball_url)
+        logging.info('extracting %s to target', tarball_url)
 
         with image_partition_mounted(self._sd_image, partition) as mntdir:
             tb = download_image(tarball_url, self.context, decompress=False)
@@ -361,8 +389,12 @@ class FastModelTarget(Target):
                 os.chmod(os.path.join(root, f), 0o777)
 
     def power_off(self, proc):
-        super(FastModelTarget, self).power_off(proc)
+        if self._uefi_vars is not None:
+            logging.info('Requesting graceful shutdown')
+            self._sim_proc.kill(signal.SIGTERM)
+            self._sim_proc.wait()
         finalize_process(self._sim_proc)
+        super(FastModelTarget, self).power_off(proc)
         self._sim_proc = None
 
     def _create_rtsm_ostream(self, ofile):
@@ -380,6 +412,10 @@ class FastModelTarget(Target):
         DrainConsoleOutput(proc=self._sim_proc).start()
 
     def power_on(self):
+        if self.config.bridged_networking:
+            self._interface_name = os.path.basename(self._scratch_dir)
+            if not self._bridge_configured:
+                self._config_network_bridge(self.config.bridge_interface, self._interface_name)
         if self._ramdisk_boot and self._booted:
             self.proc.sendline('export PS1="%s"'
                                % self.tester_ps1,
@@ -403,11 +439,12 @@ class FastModelTarget(Target):
         sim_cmd = '%s %s' % (self.config.simulator_command, options)
         sim_cmd = sim_cmd.format(
             AXF=self._axf, IMG=self._sd_image, KERNEL=self._kernel,
-            DTB=self._dtb, INITRD=self._initrd, UEFI=self._uefi, BL1=self._bl1)
+            DTB=self._dtb, INITRD=self._initrd, UEFI=self._uefi, BL1=self._bl1,
+            UEFI_VARS=self._uefi_vars, INTERFACE=self._interface_name)
 
         # the simulator proc only has stdout/stderr about the simulator
         # we hook up into a telnet port which emulates a serial console
-        logging.info('launching fastmodel with command %r' % sim_cmd)
+        logging.info('launching fastmodel with command %r', sim_cmd)
         # The base and cortex models must be invoked from the output directory
         # to ensure that the secure firmware can chainload one another.
         # bl1 -> bl2 -> bl3.
@@ -415,7 +452,7 @@ class FastModelTarget(Target):
         self._sim_proc = self.context.spawn(sim_cmd, cwd=odir, timeout=1200)
         self._sim_proc.expect(self.PORT_PATTERN, timeout=300)
         self._serial_port = self._sim_proc.match.groups()[0]
-        logging.info('serial console port on: %s' % self._serial_port)
+        logging.info('serial console port on: %s', self._serial_port)
 
         match = self._sim_proc.expect(["ERROR: License check failed!",
                                        "Simulation is started"])
